@@ -7,7 +7,7 @@ use Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(spawn receive msg snd quit wt snd_wt listener open_node);
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 use Carp;
 use IO::Select;
@@ -42,6 +42,8 @@ my %w_bufs = ();
 my %pack   = ();
 my %unpack = ();
 
+my %closed = ();
+
 my $need_reset = 0;
 
 my $blksize = 1024 * 16;
@@ -74,6 +76,7 @@ sub snd($$;@) {
 	$vpid = $self_parent_vpid if $vpid == 0;
 	$DEBUG and print "Send message '$msg' from $self_vpid to $vpid vpid in $self_vpid (\$\$=$$) with args: ", join(", ", @args), ".\n";
 	push @{$snd{$vpid}}, [$self_vpid, $vpid, $msg, \@args];
+	$closed{$vpid} = 1 if $msg eq "close" or $msg eq "exit";
 	return 1;
 }
 
@@ -199,6 +202,8 @@ sub receive(&) {
 			%pack   = ();
 			%unpack = ();
 
+			%closed = ();
+
 			$need_reset = 0;
 
 			$self_parent_fh   = $parent;
@@ -379,13 +384,14 @@ sub ipc_loop(;$$) {
 				if (my $node_vpid = $node{$fh}) {
 					delete $node{$fh};
 					if ($msg{NODE_CLOSED}) {
-						$msg{NODE_CLOSED}->($node_vpid, $fh->connected ? 1 : 0);
+						$msg{NODE_CLOSED}->($node_vpid, $fh->connected ? 1 : 0) unless $closed{$vpid};
 					}
 				} else {
 					if ($msg{SPAWN_CLOSED}) {
-						$msg{SPAWN_CLOSED}->($vpid);
+						$msg{SPAWN_CLOSED}->($vpid) unless $closed{$vpid};
 					}
 				}
+				delete $closed{$vpid};
  				close $fh;
 				if ($self_parent_fh and $self_parent_fh eq $fh) {
 					unless (defined $waited_vpid and defined $waited_msg) {
@@ -416,7 +422,7 @@ sub ipc_loop(;$$) {
 		} else {
 			while (my $rcv = shift @rcv) {
 				my ($from, $msg, $args)= @{$rcv};
-				$msg{$msg}->($from, @$args);
+				$msg{$msg}->($from, @$args) unless $closed{$from};
 			}
 		}
 		if ($need_reset) {
@@ -534,6 +540,9 @@ To detect spawn closing message SPAWN_CLOSED handler should be defined:
  	...
  };
 
+Note, for your convenience "close" and "exit" messages are special.
+If a process is sent a "close" or "exit" message, then any messages from it are ignored, SPAWN_CLOSED and NODE_CLOSED too.
+
 =head2 Dataflow programming
 
 Sometimes you may need to get additional information from other processes and only then continue the message processing. For this you may send a message with information request and then wait information getting in a proper place by subprogram wt (abbreviated "wait"), without current message processing break.
