@@ -7,7 +7,7 @@ use Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(spawn receive msg snd quit wt snd_wt listener open_node);
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 use Carp;
 use Event;
@@ -39,6 +39,8 @@ my %w_bufs = ();
 my %pack   = ();
 my %unpack = ();
 
+my %closed = ();
+
 my %fh2ww = ();
 
 my ($waited_vpid, $waited_msg, @waited_rv);
@@ -73,6 +75,7 @@ sub snd($$;@) {
 	$vpid = $self_parent_vpid if $vpid == 0;
 	$DEBUG and print "Send message '$msg' from $self_vpid to $vpid vpid in $self_vpid (\$\$=$$) with args: ", join(", ", @args), ".\n";
 	push @{$snd{$vpid}}, [$self_vpid, $vpid, $msg, \@args];
+	$closed{$vpid} = 1 if $msg eq "close" or $msg eq "exit";
 	w_event_cb_reg($vpid);
 	return 1;
 }
@@ -213,6 +216,8 @@ sub receive(&) {
 			%pack   = ();
 			%unpack = ();
 
+			%closed = ();
+
 			%fh2ww = (); 
 
 			($waited_vpid, $waited_msg, @waited_rv) = ();
@@ -389,15 +394,16 @@ sub r_event_cb {
 				if (my $node_vpid = $node{$fh}) {
 					delete $node{$fh};
 					if ($msg{NODE_CLOSED}) {
-						$msg{NODE_CLOSED}->($node_vpid, $fh->connected ? 1 : 0);
+						$msg{NODE_CLOSED}->($node_vpid, $fh->connected ? 1 : 0) unless $closed{$vpid};
 						w_event_cb_reg();
 					}
 				} else {
 					if ($msg{SPAWN_CLOSED}) {
-						$msg{SPAWN_CLOSED}->($vpid);
+						$msg{SPAWN_CLOSED}->($vpid) unless $closed{$vpid};
 						w_event_cb_reg();
 					}
 				}
+				delete $closed{$vpid};
  				close $fh;
 				if ($self_parent_fh and $self_parent_fh eq $fh) {
 					unless (defined $waited_vpid and defined $waited_msg) {
@@ -428,7 +434,7 @@ sub r_event_cb {
 		} else {
 			while (my $rcv = shift @rcv) {
 				my ($from, $msg, $args)= @{$rcv};
-				$msg{$msg}->($from, @$args);
+				$msg{$msg}->($from, @$args) unless $closed{$from};
 				w_event_cb_reg();
 			}
 		}
